import json
import os

is_debug = True if os.environ.get("dev", default='false') == "true" else False

width_for_4k="3840"
width_for_2k="2560"

def get_scale(val) -> str:
  if val == width_for_4k:
    return "940"
  elif val == width_for_2k:
    return "740"
  else:
    return "UNKNOWN"

def  get_x(val) -> str:
  if val == width_for_4k:
    return "420"
  elif val == width_for_2k:
    return "350"
  else:
    return "UNKNOWN"

def get_y(val) -> str:
  if val == width_for_4k:
    return "1500"
  elif val == width_for_2k:
    return "980"
  else:
    return "UNKNOWN"


class MP4View:
  def __init__(self, map):
    self.map = map

  def get_width(self)->int:
    return self.map["width"]

  def get_height(self)->int:
    return self.map["height"]

  def get_codec(self)->str:
    return self.map["codec_name"]

  def get_codec_long_name(self)->str:
    return self.map["codec_long_name"]

def mp4view(f_name)->MP4View:
  abs_file_name = str(os.path.abspath(f_name))
  cmd = f"ffprobe -v error -select_streams v -show_streams -of json {abs_file_name}"
  out = os.popen(cmd).read()
  d = json.loads(out)
  return MP4View(d["streams"][0])


# if __name__ == "__main__":
#   if sys.argv[1] == "scale":
#     print(get_scale())
#   elif sys.argv[1] == "x":
#     print(get_x())
#   elif sys.argv[1] == "y":
#     print(get_y())
#   else:
#     sys.exit(1)



import argparse
import json
import sys
from thinkware_file_extractor_xethhung12.script_gen import gen_merge_script, gen_copy_script

import thinkware_file_extractor_xethhung12 as thinkwareExtractor


def main():
    parser = argparse.ArgumentParser(
        prog='thinkware_file_extractor_xethhung12',
        description='extract thinkware data',
    )

    level_1_subparser=parser.add_subparsers(help="utils", dest="level1")
    level_1_subparser_utils_parser = level_1_subparser.add_parser(
        "utils", description="utils"
    )
    utils_parser_level_2=level_1_subparser_utils_parser.add_subparsers(help="when-x", dest="level2")

    utils_parser_level_2_when_x=utils_parser_level_2.add_parser("when-x")
    utils_parser_level_2_when_x.add_argument(
        "--value","-v", type=str, help="x value"
    )

    utils_parser_level_2_when_x=utils_parser_level_2.add_parser("when-y")
    utils_parser_level_2_when_x.add_argument(
        "--value","-v", type=str, help="y value"
    )

    utils_parser_level_2_scale=utils_parser_level_2.add_parser("scale")
    utils_parser_level_2_scale.add_argument(
        "--value","-v", type=str, help="scale value"
    )

    level_1_subparser_mp4_parser = level_1_subparser.add_parser(
        "mp4", description="mp4 related"
    )
    level_1_subparser_mp4_parser_level_2 = level_1_subparser_mp4_parser.add_subparsers(help="mp4 related", dest="level2")
    level_1_subparser_mp4_parser_level_2_all_parser=level_1_subparser_mp4_parser_level_2.add_parser("all")
    level_1_subparser_mp4_parser_level_2_all_parser.add_argument(
        "--file-name","-f", type=str, help="view file"
    )

    level_1_subparser_mp4_parser_level_2_width_parser=level_1_subparser_mp4_parser_level_2.add_parser("width")
    level_1_subparser_mp4_parser_level_2_width_parser.add_argument(
        "--file-name","-f", type=str, help="view file"
    )

    level_1_subparser_mp4_parser_level_2_height_parser=level_1_subparser_mp4_parser_level_2.add_parser("height")
    level_1_subparser_mp4_parser_level_2_height_parser.add_argument(
        "--file-name","-f", type=str, help="view file"
    )

    level_1_subparser_mp4_parser_level_2_codec_parser=level_1_subparser_mp4_parser_level_2.add_parser("codec")
    level_1_subparser_mp4_parser_level_2_codec_parser.add_argument(
        "--file-name","-f", type=str, help="view file"
    )

    level_1_subparser_script_gen_parser = level_1_subparser.add_parser(
        "script-gen", description="generate scripts"
    )
    level_1_subparser_script_gen_parser_level_2=level_1_subparser_script_gen_parser.add_subparsers(help="merge-script", dest="level2")
    level_1_subparser_script_gen_parser_level_2_parser=level_1_subparser_script_gen_parser_level_2.add_parser("merge-script")
    level_1_subparser_script_gen_parser_level_2_parser.add_argument(
        "--source", type=str, help="source path"
    )
    level_1_subparser_script_gen_parser_level_2_parser.add_argument(
        "--dest", type=str, help="destination path"
    )
    level_1_subparser_script_gen_parser_level_2_parser.add_argument(
        "--prefix", type=str, help="prefix of file", choices = ["EVT", "REC", "MOT", "PAK"]
    )
    level_1_subparser_script_gen_parser_level_2_parser.add_argument(
        "--mode", type=str, help="front or rear", choices = ["F", "R"]
    )
    level_1_subparser_script_gen_parser_level_2_parser.add_argument(
        "--dest-file-name", type=str, help="destination file name", default="merge"
    )

    level_1_subparser_script_gen_parser_level_2_copy_script_parser=level_1_subparser_script_gen_parser_level_2.add_parser("copy-script")
    level_1_subparser_script_gen_parser_level_2_copy_script_parser.add_argument(
        "--source", type=str, help="source path"
    )
    level_1_subparser_script_gen_parser_level_2_copy_script_parser.add_argument(
        "--dest", type=str, help="destination path"
    )

    x = parser.parse_args(sys.argv[1:])
    if x.level1 == "utils":
        if x.level2 == "when-x":
            val = x.value
            return print(thinkwareExtractor.utils.get_x(val))
        elif x.level2 == "when-y":
            val = x.value
            return print(thinkwareExtractor.utils.get_y(val))
        elif x.level2 == "scale":
            val = x.value
            return print(thinkwareExtractor.utils.get_scale(val))
        else:
            print(x)
    elif x.level1 == "mp4":
        if x.level2 == "all":
            print(json.dumps(thinkwareExtractor.utils_tools.mp4view(x.file_name).map, indent=2))
        elif x.level2 == "width":
            print(thinkwareExtractor.utils_tools.mp4view(x.file_name).get_width())
        elif x.level2 == "height":
            print(thinkwareExtractor.utils_tools.mp4view(x.file_name).get_height())
        elif x.level2 == "codec":
            print(thinkwareExtractor.utils_tools.mp4view(x.file_name).get_codec())
        else:
            print(x)
    elif x.level1 == "script-gen":
        if x.level2 == "merge-script":
            print(gen_merge_script(x.source, x.dest, x.prefix, x.mode, x.dest_file_name if x.dest_file_name is not None else "merge"))
        elif x.level2 == "copy-script":
            print(gen_copy_script(x.source, x.dest))
        else:
            print(x)
    else:
        print(x)


if __name__ == '__main__':
    main()



import os
from os.path import abspath, exists, isdir

from thinkware_file_extractor_xethhung12 import is_debug


def gen_merge_script(source, dest, prefix, suffix, dest_file_name):
    source = abspath(source)
    source = source[0:-1] if source.endswith("/") else source
    dest = abspath(dest)
    dest = dest[0:-1] if dest.endswith("/") else dest
    if is_debug:
        print(f"[debug]mkdirs: {dest}")
    os.makedirs(dest, exist_ok=True)
    dest_file=f"{dest}/{dest_file_name}"
    dest_mp4_file=f"{dest}/{dest_file_name}.mp4"

    replaced_source=source.replace("/","\\/")+"\\/"
    return f"""


##### Generate merge script 
mkdir -p {dest}
rm -f {dest_file} {dest_mp4_file}
ls -al {source} | grep -o {prefix}_.*_{suffix}.MP4 | sed 's/^/file {replaced_source}/' > {dest_file}
ffmpeg -f concat -safe 0 -i {dest_file} -vcodec copy -acodec copy {dest_mp4_file}
"""

def gen_copy_script(source, dest):
    if not exists(source):
        raise FileNotFoundError
    if not isdir(source):
        raise NotADirectoryError

    def remove_last_slash(p):
        return p[0:-1] if p.endswith("/") else p

    source = remove_last_slash(str(abspath(source)))
    source_cont_rec=remove_last_slash(f"{source}/cont_rec")
    source_evt_rec=remove_last_slash(f"{source}/evt_rec")
    source_manual_rec=remove_last_slash(f"{source}/manual_rec")
    source_motion_timelapse_rec=remove_last_slash(f"{source}/motion_timelapse_rec")
    source_parking_rec=remove_last_slash(f"{source}/parking_rec")

    os.makedirs(dest,exist_ok=True)
    dest = remove_last_slash(str(abspath(dest)))
    dest_cont_rec=remove_last_slash(f"{dest}/cont_rec")
    dest_evt_rec=remove_last_slash(f"{dest}/evt_rec")
    dest_manual_rec=remove_last_slash(f"{dest}/manual_rec")
    dest_motion_timelapse_rec=remove_last_slash(f"{dest}/motion_timelapse_rec")
    dest_parking_rec=remove_last_slash(f"{dest}/parking_rec")

    def script_from_to(src, dest):
        return f"""mkdir -p {dest}
echo 'copy from `{src}` to `{dest}`'
rm -fr {dest}
cp -r {src} {dest}
echo "origin file count: $(ls -al {src} | wc -l)"
echo "dest file count: $(ls -al {dest} | wc -l)"
"""

    script_gen = "\n".join([script_from_to(s, d) for s,d in [
        [source_cont_rec, dest_cont_rec],
        [source_evt_rec, dest_evt_rec],
        [source_manual_rec, dest_manual_rec],
        [source_motion_timelapse_rec, dest_motion_timelapse_rec],
        [source_parking_rec, dest_parking_rec]
    ]])
    return script_gen




